;
; ------------------------------------------------------------
;
;   PureBasic - DEXVeeCADNetList
;   Create a netlist for VeeCAD from an AutoTRAX Netlist.
;   A default Outline for each component is created and can be editted
;   Various pins are remapped by default. These can be adjusted as required.
;   Unused components removed each run
;
; ------------------------------------------------------------
;

#Window     = 0
#TreeGadget = 0
#XML        = 0
Global closerequired=0, namepending=0, indexpending=0
Global netlist.s,FileName$

UseSQLiteDatabase()
 
 Procedure DatabaseUpdateOrInsert(Database, component$,outline$,value$)
   insert$="INSERT INTO components (component) VALUES ('"+component$+"')"
   Result = DatabaseUpdate(Database, insert$)
   
   insert$="INSERT INTO netlist (component, outline,value) VALUES ('"+component$+"', '"+outline$+"','"+value$+"')"
   Result = DatabaseUpdate(Database, insert$)
   ;Result = DatabaseUpdate(Database, "INSERT INTO netlist (component, outline,value) VALUES ('U4', 'DIP8','xxxx')")
   If Result = 0
    ; Debug DatabaseError()
    ; Debug insert$
     update$="UPDATE netlist SET value='"+value$+"' WHERE component='"+component$+"'"
     Result = DatabaseUpdate(Database, update$)
     If Result = 0
        ;Debug DatabaseError()
       ; Debug update$
     EndIf
     
   EndIf
   
   ProcedureReturn Result
 EndProcedure
 
  Procedure DatabaseUpdateOrInsertPinMap(Database, component$,pin$,newpin$)
   insert$="INSERT INTO pinmap (component, pin,newpin) VALUES ('"+component$+"', '"+pin$+"','"+newpin$+"')"
   If DatabaseUpdate(Database, insert$)
      ; AddGadgetItem(10, -1, "Insert OK for "+insert$ ) 
   Else
     ;AddGadgetItem(10, -1, "Insert failed for "+insert$ )
       
   EndIf
   
   ProcedureReturn Result
 EndProcedure
  Procedure ResetComponentsDatabase()
   
    ; Delete all netlist entries
    DatabaseUpdate(2, "DELETE FROM components")
    ;MessageRequester("Information", "netlist database cleared-redo the file to set defaults value", #PB_MessageRequester_Ok)
 EndProcedure
 
 Procedure ResetNetlistDatabase()
   
    ; Delete all netlist entries
    DatabaseUpdate(2, "DELETE FROM netlist")
    ;MessageRequester("Information", "netlist database cleared-redo the file to set defaults value", #PB_MessageRequester_Ok)
 EndProcedure
  
Procedure ResetPinMapDatabase()
   
    ; Delete all netlist entries
    DatabaseUpdate(2, "DELETE FROM pinmap")
    ;MessageRequester("Information", "netlist database cleared-redo the file to set defaults value", #PB_MessageRequester_Ok)
EndProcedure

; This procedure fills our TreeGadget, by adding the current node
; and then exploring all childnodes by recursively calling itself.
; data is output from in this loop as well
Procedure FillTree(*CurrentNode, CurrentSublevel)
  
  ; Ignore anything except normal nodes. See the manual for
  ; XMLNodeType() for an explanation of the other node types.
  ;
  If XMLNodeType(*CurrentNode) = #PB_XML_Normal
  
    ; Add this node to the tree. Add name and attributes
    ;
    Text$ = GetXMLNodeName(*CurrentNode) + " (Attributes: "
        
    If ExamineXMLAttributes(*CurrentNode)
      While NextXMLAttribute(*CurrentNode)
        Text$ + XMLAttributeName(*CurrentNode) + "=" + Chr(34) + XMLAttributeValue(*CurrentNode) + Chr(34) + " "
        If  (GetXMLNodeName(*CurrentNode)="Part")
         If (XMLAttributeName(*CurrentNode)="reference") 
           netlist=netlist+"["+#CRLF$
           netlist=netlist+XMLAttributeValue(*CurrentNode)+#CRLF$
         
           currentcomponent$=XMLAttributeValue(*CurrentNode)
          
         EndIf 
         If (XMLAttributeName(*CurrentNode)="value") 
          
           currentvalue$=XMLAttributeValue(*CurrentNode)
           ;Debug "component+value="+currentcomponent$+" "+currentvalue$
           defaultoutline$="BOX1_1"
           ; capacitors
           If (Left(currentcomponent$,1)="C"): defaultoutline$="BOX1_1":EndIf
          ; If (Left(currentcomponent$,1)="C" And FindString("100uF/330uF/470uF/",currentvalue$)>0): defaultoutline$="RA1_2":EndIf
          ; If (Left(currentcomponent$,1)="C" And FindString("1000uF/2200uF/1,000"+Chr(181)+"F/2,200uF",currentvalue$)>0): defaultoutline$="RA1_4":EndIf
          ; If (Left(currentcomponent$,1)="C" And FindString("1000uF 2200uF 1,000"+Chr(181)+"F 2,200uF",currentvalue$)>0)
          ;    AddGadgetItem(10, -1, "FOUND"+currentvalue$)
          ; EndIf
           ;AddGadgetItem(10, -1, "FOUND"+"1000uF 2200uF 1,000"+Chr(181)+"F 2,200uF")
           ;Resistors
           If (Left(currentcomponent$,1)="R"): defaultoutline$="AX3_1N":EndIf
           If (Left(currentcomponent$,2)="VR"): defaultoutline$="TRIM5":EndIf
           
           ; Diodes
           ;  2 -->|-- 1  Anode is pin 1, Cathode is pin 2
           If (Left(currentcomponent$,1)="D"): defaultoutline$="AX2_1R":EndIf  ; Pin 2 has the marking. so need AX2_IR   (reverse marking)
           
           ; Transistors
           If (Left(currentcomponent$,1)="Q"): defaultoutline$="TO92":EndIf
           
           ; Connectors
           If (Left(currentcomponent$,1)="J"): defaultoutline$="SIP6":EndIf
           If (Left(currentcomponent$,1)="J" And FindString("E64*E100*LCD40",currentvalue$)>0): defaultoutline$=currentvalue$:EndIf
           If (Left(currentcomponent$,1)="J" And FindString(currentvalue$,"5x1")>0): defaultoutline$="SIP5":EndIf
           If (Left(currentcomponent$,1)="J" And FindString(currentvalue$,"4x1")>0): defaultoutline$="SIP4":EndIf
           If (Left(currentcomponent$,1)="J" And FindString(currentvalue$,"3x1")>0): defaultoutline$="SIP3":EndIf
           If (Left(currentcomponent$,1)="J" And FindString(currentvalue$,"2x1")>0): defaultoutline$="SIP2":EndIf
           
           ;Relays and Connectors
           If (Left(currentcomponent$,1)="K"): defaultoutline$="G6K-2F":EndIf
           If (Left(currentcomponent$,3)="BNC"): defaultoutline$="BNC-U":EndIf
          
           ;ICs
           If (Left(currentcomponent$,1)="U"): defaultoutline$="DIP8":EndIf
           If (Left(currentcomponent$,1)="U" And currentvalue$="4066"): defaultoutline$="DIP14":EndIf
           If (Left(currentcomponent$,1)="U" And currentvalue$="20x2Header"): defaultoutline$="IDC40":EndIf
           If (Left(currentcomponent$,1)="U" And FindString("E64*E100*LCD40",currentvalue$)>0): defaultoutline$=currentvalue$:EndIf
           If (Left(currentcomponent$,1)="U" And FindString(currentvalue$,"LM2940")>0): defaultoutline$="TO220":EndIf
           If (Left(currentcomponent$,1)="U" And FindString(currentvalue$,"LM3940")>0): defaultoutline$="TO220":EndIf
           If (Left(currentcomponent$,1)="U" And FindString(currentvalue$,"TC1264")>0): defaultoutline$="TO220":EndIf
           
           ;Mapping based on specific values
                 
           If (FindString("BC817*BC807*",currentvalue$)>0): defaultoutline$="SOT23":EndIf
           
           ;If (FindString(currentvalue$,"BC817")>0): defaultoutline$="SOT23":EndIf
           
         ; FindString(thisline$, "<Part reference=")>0 
           ;Insert into the netlist database. outline will not update if already exists
           DatabaseUpdateOrInsert(2, currentcomponent$,defaultoutline$,currentvalue$)
           If DatabaseQuery(2, "SELECT * FROM netlist WHERE component='"+currentcomponent$+"'") ; Get the record
             While NextDatabaseRow(2) ; Loop for each records should be only one
              
               outline$=GetDatabaseString(2, 1)
             Wend
              FinishDatabaseQuery(2)
           EndIf
            
            netlist=netlist+outline$+#CRLF$ 
            netlist=netlist+XMLAttributeValue(*CurrentNode)+#CRLF$
            netlist=netlist+"]"+#CRLF$
   
         EndIf  
       EndIf 
       If  (GetXMLNodeName(*CurrentNode)="Node")
         If (XMLAttributeName(*CurrentNode)="index") 
           
           If (closerequired=1)
             netlist=netlist+")"+#CRLF$
             closerequired=0
           EndIf
           closerequired=1
           netlist=netlist+"("+#CRLF$
           netlist=netlist+XMLAttributeValue(*CurrentNode)
           indexpending=1
         EndIf  
         If (XMLAttributeName(*CurrentNode)="name") 
           netlist=netlist+"-"+XMLAttributeValue(*CurrentNode)+#CRLF$
           
           namepending=0
           indexpending=0

         EndIf 
           
       EndIf 
       If  (GetXMLNodeName(*CurrentNode)="Terminal")
           
         If (XMLAttributeName(*CurrentNode)="Device") 
           If (namepending=1 Or indexpending=1)
             netlist=netlist+#CRLF$
             namepending=0
             indexpending=0
           EndIf  
           netlist=netlist+XMLAttributeValue(*CurrentNode)
           thisnode$=XMLAttributeValue(*CurrentNode)
         EndIf  
         If (XMLAttributeName(*CurrentNode)="pin") 
           pinname$=XMLAttributeValue(*CurrentNode)
           newpin$=pinname$
           Select pinname$ 
            ;resistor / capacitor   
            Case "+"
              newpin$="1"
              ; AddGadgetItem(10, -1, thisnode$ +" pin-" + pinname$ +" mapped as pin-" + newpin$)
            Case "-"
              newpin$="2"
             ; AddGadgetItem(10, -1, thisnode$ +" pin-" + pinname$ +" mapped as pin-" + newpin$)
            ;diode  
            Case "A"
              newpin$="1"
             ; AddGadgetItem(10, -1, thisnode$ +" pin-" + pinname$ +" mapped as pin-" + newpin$)
            Case "K"
              newpin$="2"
             ; AddGadgetItem(10, -1, thisnode$ +" pin-" + pinname$ +" mapped as pin-" + newpin$)
            ;transistor  
            Case "C"
              newpin$="1"
             ; AddGadgetItem(10, -1, thisnode$ +" pin-" + pinname$ +" mapped as pin-" + newpin$)
            Case "B"
              newpin$="2"
             ; AddGadgetItem(10, -1, thisnode$ +" pin-" + pinname$ +" mapped as pin-" + newpin$)
            Case "E"
              newpin$="3"
              ;AddGadgetItem(10, -1, thisnode$ +" pin-" + pinname$ +" mapped as pin-" + newpin$)
                    
             Case "Anode"
               newpin$="1"
             Case "Cathode"
               newpin$="2" 
               
               
          EndSelect
           ;Insert into the pinmap database. newpin will not update if already exists
             DatabaseUpdateOrInsertPinMap(2, thisnode$,pinname$,newpin$)
             If DatabaseQuery(2, "SELECT * FROM pinmap WHERE component='"+thisnode$+"' and pin='"+pinname$+"'") ; Get the record
               While NextDatabaseRow(2) ; Loop for each records should be only one
               ;Debug GetDatabaseString(2, 0) ; Display the content of the first field 
               newpin$=GetDatabaseString(2, 2)
              Wend
              FinishDatabaseQuery(2)
            EndIf
         
          
           netlist=netlist+"-"+newpin$+#CRLF$
         
         EndIf 
         
        EndIf 
      Wend
      
    EndIf
     
    Text$ + ")"
    
    AddGadgetItem(#TreeGadget, -1, Text$, 0, CurrentSublevel)
        
        
    ; Now get the first child node (if any)
    ;    
    *ChildNode = ChildXMLNode(*CurrentNode)
    
    ; Loop through all available child nodes and call this procedure again
    ;
    While *ChildNode <> 0
      FillTree(*ChildNode, CurrentSublevel + 1)      
      *ChildNode = NextXMLNode(*ChildNode)
    Wend        
  
  EndIf
   
 
EndProcedure

 
 
 ;  **********************************************The program starts Here**********************************
 ;  ********************************************************************************************************
 
Procedure ProcessFile()
  ClearGadgetItems(5)
  ClearGadgetItems(10)
  ClearGadgetItems(11)
  ClearGadgetItems(12)
 ;FileName$ = OpenFileRequester("Choose XML file...", "", "XML files (*.xml)|*.xml|All files (*.*)|*.*", 0)
;FileName$ = OpenFileRequester("Choose NET file...", "", "Net files (*.net)|*.net|All files (*.*)|*.*", 0)
If FileName$ <> ""
  DBFilename$=ReplaceString(FileName$,"net","sqlite")
  If (FileSize(DBFilename$)=-1)
    
    If CreateFile(2, DbFilename$)
      AddGadgetItem(10, -1, "Created"+DbFilename$)
      
      CloseFile(2)
    Else
     
       AddGadgetItem(10, -1, "Can't create database !")
    EndIf
     
  
     If OpenDatabase(2, DbFilename$, "", "")
       AddGadgetItem(10, -1,  "Connected to "+DbFilename$)
       
       If DatabaseUpdate(2, "CREATE TABLE components (component CHAR(20))")
            AddGadgetItem(10, -1, "Table components created")
       Else 
           AddGadgetItem(10, -1, "Table components NOT created - "+ DatabaseError())

       EndIf
       
       
       If DatabaseUpdate(2, "CREATE TABLE netlist (component CHAR(20),outline CHAR(30),value CHAR(30))")
        
         AddGadgetItem(10, -1, "Table netlist created")
       Else 
         
         AddGadgetItem(10, -1, "Table netlist NOT created")
       EndIf
      
      If DatabaseUpdate(2, "CREATE UNIQUE INDEX netlistidx ON netlist (component)")
        AddGadgetItem(10, -1, "Index netlistidx created")
      Else 
         AddGadgetItem(10, -1, "Index netlistidx NOT created")
      EndIf
      If DatabaseUpdate(2, "CREATE TABLE pinmap (component CHAR(20),pin CHAR(2),newpin CHAR(2))")
        
        AddGadgetItem(10, -1, "Table pinmap created")
      Else
        AddGadgetItem(10, -1, "Table pinmap NOT created")
      EndIf
      
      If DatabaseUpdate(2, "CREATE UNIQUE INDEX pinmapidx ON pinmap (component,pin)")
           AddGadgetItem(10, -1, "Index pinmapidx created")
      Else
           AddGadgetItem(10, -1, "Index pinmapidx NOT created")
      EndIf
    Else
     
       AddGadgetItem(10, -1, "Can't open database ! "+DbFilename$)
    EndIf  
    
  Else
      If OpenDatabase(2, DbFilename$, "", "")
        AddGadgetItem(10, -1, "Connected to "+DbFilename$)
      ; If DatabaseUpdate(2, "CREATE TABLE components (component CHAR(20))")
      ;      AddGadgetItem(10, -1, "Table components created")
      ; Else 
      ;     AddGadgetItem(10, -1, "Table components NOT created - "+ DatabaseError())

      ; EndIf
        
      Else
     
         AddGadgetItem(10, -1, "Can't open existing database ! "+DbFilename$)
      EndIf  
        
  EndIf 
    
  ResetComponentsDatabase()
    
  ;enable menus now file is selected  
  DisableMenuItem(0, 2, 0)
  DisableMenuItem(0, 3, 0)
  DisableMenuItem(0, 4, 0)
  
    ;If DatabaseQuery(2, "SELECT * FROM netlist")
    ;    While NextDatabaseRow(2)
             ;AddGadgetItem(10, -1, GetDatabaseString(2, 0)+" "+GetDatabaseString(2, 1)+" "+GetDatabaseString(2, 2))
    ;   Wend
    ;   FinishDatabaseQuery(2)
    ;EndIf
  
  If CreateFile(1, ReplaceString(FileName$,"net","xml"))
    
  Else
     MessageRequester("PureBasic", "Error: can't write the file", 0)
     End
  EndIf
   If ReadFile(0, FileName$)   ; if the file could be read, we continue...
     While Eof(0) = 0          ; loop as long the 'end of file' isn't reached
      thisline$=ReadString(0)
      ;Debug ReplaceString(ReadString(0), "utf-16","utf-8") ;ReadString(0)      ; display line by line in the debug window
      If thisline$<>lastline$
        If (FindString(thisline$, "<Part reference=")>0  And  FindString(thisline$,"value=")=0)
          AddGadgetItem(10, -1, "No value= in XML: "+thisline$)
          thisline$=ReplaceString(thisline$,"/>","value="+Chr(34)+"Value"+Chr(34) +" />")
        EndIf 
        
        WriteStringN(1, ReplaceString(thisline$, "utf-16","utf-8"))
      Else  
        AddGadgetItem(10, -1, "Duplicated Removed: "+thisline$)
      EndIf
      lastline$=thisline$
    Wend
    CloseFile(0)               ; close the previously opened file
    CloseFile(1)               ; close the previously opened file
  Else
    MessageRequester("Information","Couldn't open the file!")
  EndIf
  If LoadXML(#XML, ReplaceString(FileName$,"net","xml"))

    ; Note: 
    ;   The LoadXML() succeed if the file could be read. This does not mean that
    ;   there was no error in the XML though. To check this, XMLStatus() can be
    ;   used.
    ;
    ; Display an error message if there was a markup error
    ;
    If XMLStatus(#XML) <> #PB_XML_Success
      Message$ = "Error in the XML file:" + Chr(13)
      Message$ + "Message: " + XMLError(#XML) + Chr(13)
      Message$ + "Line: " + Str(XMLErrorLine(#XML)) + "   Character: " + Str(XMLErrorPosition(#XML))
      MessageRequester("Error", Message$)
    EndIf
    ;
    


    ;OpenConsole()                                  ; First we must open a console
   ; ConsoleTitle ("VeeCAD NetList:")               ; Now we can give the opened console a Titlename ;)  
    netlist=""
    
  Else
    MessageRequester("Error", "The file cannot be opened.")
  EndIf 
EndIf  

 ; Get the main XML node, and call the FillTree() procedure with it
 ;
      ClearGadgetItems(0)

      *MainNode = MainXMLNode(#XML)      
      If *MainNode
        FillTree(*MainNode, 0)
      EndIf
      ;
    
      ; Expand all nodes for a nicer view
      ;
      For i = 0 To CountGadgetItems(#TreeGadget) - 1
        SetGadgetItemState(#TreeGadget, i, #PB_Tree_Expanded)
      Next i
      netlist=netlist+")"+#CRLF$
      SetClipboardText(netlist) ; put the output onto the clipboard
      
   
      
       ; 
      AddGadgetItem(10, -1, "Creating VeeCAD Netlist: "+ReplaceString(FileName$,".net","VeeCad.net"))
      ; 
     
      AddGadgetItem(11, -1, netlist)
      If CreateFile(2, ReplaceString(FileName$,".net","VeeCad.net"))
        WriteStringN(2,netlist,#PB_Ascii   )
        CloseFile(2) 
      Else
       MessageRequester("PureBasic", "Error: can't write the file", 0)
       End
     EndIf
     
         
       ;If DatabaseQuery(2, "SELECT component FROM components") ; Get the record
       ;       While NextDatabaseRow(2) ; Loop for each records should be only one
                ;Debug GetDatabaseString(2, 0) ; Display the content of the first field 
                ; newpin$=GetDatabaseString(2, 2)
       ;         AddGadgetItem(10, -1, "used component "+GetDatabaseString(2, 0)) 
       ;;       Wend
      ;        FinishDatabaseQuery(2)
      ; Else
      ;        AddGadgetItem(10, -1, "Table components select * - "+ DatabaseError())   
      ;  EndIf
     
      AddGadgetItem(10, -1, "Removing unused components")   
       DatabaseUpdate(2, "DELETE FROM netlist WHERE component NOT IN (SELECT component from components)")
       DatabaseUpdate(2, "DELETE FROM pinmap WHERE component NOT IN (SELECT component from components)")
        
       ; Fill Up the outline list. Notice than the column are separated by Chr(10) (NewLine) character
       If DatabaseQuery(2, "SELECT * FROM netlist ORDER BY component")
         While NextDatabaseRow(2)
        
          AddGadgetItem(5, -1, GetDatabaseString(2, 0)+Chr(10)+GetDatabaseString(2, 2)+Chr(10)+GetDatabaseString(2,1))
         Wend
         FinishDatabaseQuery(2)
       EndIf
      
       SetActiveGadget(5) 
       SetGadgetState(5, -1  )
       ;SetGadgetItemState(5, 0 ,#PB_ListIcon_Selected )
       
       AddGadgetItem(10, -1, "List pin map database: ")
       If DatabaseQuery(2, "SELECT * FROM pinmap order by component,pin")
         While NextDatabaseRow(2)
           
           AddGadgetItem(12, -1, GetDatabaseString(2, 0)+Chr(10)+GetDatabaseString(2, 1)+Chr(10)+GetDatabaseString(2,2))
         Wend
         FinishDatabaseQuery(2)
       EndIf
       
       SetActiveGadget(12) 
       SetGadgetState(12, -1 ) 
       
        
      ; CloseConsole()

EndProcedure 
     
Procedure SelectAndProcessFile()
  FileName$ = OpenFileRequester("Choose NET file...", "", "Net files (*.net)|*.net|All files (*.*)|*.*", 0)
  If FileName$ <> ""
     SetWindowTitle(0,"AutoTRAX NetList for VeeCAD - "+FileName$) 
     ProcessFile()
  EndIf   
EndProcedure 
Procedure ProcessFileWithReset()
  
  If FileName$ <> ""
     ResetNetlistDatabase()
     ProcessFile()
  EndIf   
EndProcedure 
    ;EnableGraphicalConsole(1)
    ;ClearConsole()
   ; CloseConsole()
   ;CreatePopupMenu(0)
    ; MenuItem(0, "Popup !")
    ; Note:
    ;   Even if there was an error in the XML, all nodes before the error position
    ;   are still accessible, so open the window and show the tree anyway.
    ;
If OpenWindow(#Window, 0, 0, 1024, 768, "AutoTRAX NetList for VeeCAD", #PB_Window_SystemMenu|#PB_Window_ScreenCentered)
       
      ; Create the menu. The indent is very important here for a good lisibility
       ;

       If CreateMenu(0, WindowID(0))
         MenuTitle("File")
          MenuItem( 1, "Select and Process File ...")
          MenuItem( 2, "Redo Current File")
          MenuItem( 3, "Reset Outline Database")
          MenuItem(4,"Reset Pin Map Database")
          MenuBar()
         ; OpenSubMenu("Recents")
         ;   MenuItem( 4, "Pure.png")
         ;   MenuItem( 5, "Basic.jpg")
         ;   OpenSubMenu("Even more !")
         ;    MenuItem( 6, "Yeah")
         ;   CloseSubMenu()
         ;   MenuItem( 7, "Rocks.tga")
         ; CloseSubMenu()
         ;MenuBar()
         MenuItem( 8, "&Quit")

        ; MenuTitle("Edition")
        ;  MenuItem(9, "Cut")
        ;   MenuItem( 10, "Copy")
        ;   MenuItem(11, "Paste")
      
           MenuTitle("Help")
           MenuItem(12, "Help")
           MenuItem(13, "About")
          
          BindMenuEvent(0, 1, @SelectAndProcessFile())
          BindMenuEvent(0, 2, @ProcessFile())
          ;BindMenuEvent(0, 3, @ProcessFileWithReset())
          BindMenuEvent(0, 3,@ResetNetlistDatabase())
          BindMenuEvent(0, 4,@ResetPinMapDatabase())
       EndIf
       ;Disable menus until file selected
       DisableMenuItem(0, 2, 1): DisableMenuItem(0, 3, 1):DisableMenuItem(0, 4, 1)
       
       ;set up gadget to hold tree
        FrameGadget(#PB_Any, 1,  5, 355, 498, "Input file")

        TreeGadget(#TreeGadget, 3, 20, 350, 480)
        
        ; set up gadget to hold VeeCad Net File
        FrameGadget(#PB_Any, 865,  5, 155, 498,"VeeCad NetList")
        EditorGadget(11, 867, 20, 149, 480)
        
        ; set up gadget to hold log file
        FrameGadget(#PB_Any, 1,  500, 628, 245, "Log file")
        EditorGadget(10, 3, 515, 620, 225)
        
        ; setup outline display
        FrameGadget(#PB_Any, 358,  5, 270, 498, "Outline Database")
        ListIconGadget(5, 360, 20, 260, 400, "Component", 75,#PB_ListIcon_GridLines|#PB_ListIcon_FullRowSelect| #PB_ListIcon_AlwaysShowSelection  )
        AddGadgetColumn(5, 1, "Value", 80)
        AddGadgetColumn(5, 2, "Outline", 80)
        GadgetToolTip(5, "Click to edit")

        
         StringGadget(6, 380,  425, 100, 20, "", #PB_String_ReadOnly)
         StringGadget(7, 380,  450, 100, 20, "", #PB_String_ReadOnly)
         StringGadget(8, 380,  475, 100, 20, "", #PB_String_UpperCase  )
         ButtonGadget(9, 500, 475, 100, 20, "Update Outline")
         
          ; setup outline pinmap
        FrameGadget(#PB_Any, 630,  5, 230, 498, "Pin Mapping Database")
        ListIconGadget(12, 632, 20, 220, 400, "Component", 75,#PB_ListIcon_GridLines|#PB_ListIcon_FullRowSelect| #PB_ListIcon_AlwaysShowSelection  )
        AddGadgetColumn(12, 1, "Pin", 60)
        AddGadgetColumn(12, 2, "New Pin", 60)
        GadgetToolTip(12, "Click to edit")

        
         StringGadget(13, 640,  425, 100, 20, "", #PB_String_ReadOnly)
         StringGadget(14, 640,  450, 100, 20, "", #PB_String_ReadOnly)
         StringGadget(15, 640,  475, 100, 20, "", #PB_String_UpperCase  )
         ButtonGadget(16, 750, 475, 100, 20, "Update New Pin")

   

      Repeat
        ;See which window event we have
        Select WaitWindowEvent()

        Case #PB_Event_Menu
          ; Its a menu event, see which one
          Select EventMenu()  ; To see which menu has been selected
             Case 1
                ;If GetMenuItemState(0, 1) = 1  : SetMenuItemState(0, 1, 0)  : Else  : SetMenuItemState(0, 1, 1)  : EndIf
             Case 2
               ;If GetMenuItemState(0, 2) = 1  : SetMenuItemState(0, 2, 0)  : Else  : SetMenuItemState(0, 2, 1)  : EndIf
             Case 3 ;Reset Netlist database 
             Case 4 ;Reset PinMap database   
             Case 8 ; Quit
               Quit = 1
            
             Case 12 ; About
                ; MessageRequester with Yes / No buttons  (normally used for questions)
                ; (result will be shown in the following information requester)
               ; Result = MessageRequester("Title", "Please make your input:", #PB_MessageRequester_YesNo)
                a$ =GetCurrentDirectory()
               ;If Result = #PB_MessageRequester_Yes    ; pressed Yes button (Result = 6)
               ;        a$ + "Yes"
               ;Else                                    ; pressed No button (Result = 7)
               ;     a$ + "No"
               ;EndIf
               ;MessageRequester("Information", a$+#PB_Compiler_Home, #PB_MessageRequester_Ok)
               OpenHelp(GetCurrentDirectory() + "AutoTRAX Netlist for VeeCad.chm", "")
            Case 13 ; About
                ; MessageRequester with Yes / No buttons  (normally used for questions)
                ; (result will be shown in the following information requester)
               MessageRequester("AutoTRAX Netlist for VeeCad", "Version: 1.00 "+Chr(13)+"Date: 09/10/2016"+Chr(13)+"Current Directory: "+GetCurrentDirectory()+Chr(13)+"gerry"+Chr(13)+"gerry")
            
              Default
              MessageRequester("Info", "MenuItem: "+Str(EventMenu()), 0)

          EndSelect
        ; Its a gadget event, see which one  
        Case #PB_Event_Gadget
      
          Select EventGadget()
             Case 5  
               Select EventType()
                    
                 Case #PB_EventType_LeftClick        :; Debug "Click with left mouse button"
                   StringGadget(6, 380,  424, 100, 20, Str(GetGadgetState(5)), #PB_String_ReadOnly)
                   StringGadget(7, 380,  450, 100, 20, GetGadgetText(5), #PB_String_ReadOnly)
                   StringGadget(8, 380,  475, 100, 20, GetGadgetItemText(5,GetGadgetState(5),2), #PB_String_UpperCase  )
                   ButtonGadget(9, 500, 475, 100, 20, "Update Outline")

                 Case #PB_EventType_Focus  
                  
                  ; StringGadget(6, 380,  424, 100, 20, Str(GetGadgetState(5)), #PB_String_ReadOnly)
                   ;StringGadget(7, 380,  450, 100, 20, GetGadgetText(5), #PB_String_ReadOnly)
                  ; StringGadget(8, 380,  475, 100, 20, GetGadgetItemText(5,GetGadgetState(5),2), #PB_String_UpperCase  )
                  ; ButtonGadget(9, 500, 475, 100, 20, "Update Outline")
                
               
                 Case #PB_EventType_RightClick       :; Debug "Click with right mouse button"
                    ;DisplayPopupMenu(0, WindowID(0))
                 Case #PB_EventType_LeftDoubleClick  :; Debug "Double-click with left mouse button"
                                                      ;MessageRequester("Information", "Doubleclick: item"+Str(GetGadgetState(5))+", Text: "+GetGadgetText(5), 2)
                    
                 Case #PB_EventType_RightDoubleClick : ;Debug "Double-click with right mouse button"
               EndSelect

             Case 9 
               Select EventType()
                 Case #PB_EventType_LeftClick     
                   update$="UPDATE netlist SET outline='"+GetGadgetText(8)+"' WHERE component='"+GetGadgetText(7)+"'"
                   If DatabaseUpdate(2, update$)
                       AddGadgetItem(10, -1, "Update OK: "+update$)
                       SetGadgetItemText(5, Val(GetGadgetText(6)),GetGadgetText(8),2)
                       SetActiveGadget(5) 
                       SetGadgetState(5, Val(GetGadgetText(6))  )
                       SetGadgetItemState(5, Val(GetGadgetText(6)) ,#PB_ListIcon_Selected )
                   Else
                        AddGadgetItem(10, -1, "Update failed: "+update$)  
                   EndIf
                   
               EndSelect
               
             Case 12  
               Select EventType()
                    
                 Case #PB_EventType_LeftClick        :; Debug "Click with left mouse button"
                  ; StringGadget(13, 380,  424, 100, 20, Str(GetGadgetState(5)), #PB_String_ReadOnly)
                  ; StringGadget(14, 380,  450, 100, 20, GetGadgetText(5), #PB_String_ReadOnly)
                  ; StringGadget(15, 380,  475, 100, 20, GetGadgetItemText(5,GetGadgetState(5),2), #PB_String_UpperCase  )
                  ; ButtonGadget(16, 500, 475, 100, 20, "Update Outline")
                   
                   StringGadget(13, 640,  425, 100, 20, Str(GetGadgetState(12)), #PB_String_ReadOnly)  ; item selected
                   StringGadget(14, 640,  450, 100, 20, GetGadgetText(12), #PB_String_ReadOnly)
                   StringGadget(15, 640,  475, 100, 20, GetGadgetItemText(12,GetGadgetState(12),2), #PB_String_UpperCase  )
                   ButtonGadget(16, 750, 475, 100, 20, "Update New Pin")

                 Case #PB_EventType_Focus  
                  
                   ;StringGadget(6, 380,  424, 100, 20, Str(GetGadgetState(5)), #PB_String_ReadOnly)
                   ;StringGadget(7, 380,  450, 100, 20, GetGadgetText(5), #PB_String_ReadOnly)
                   ;StringGadget(8, 380,  475, 100, 20, GetGadgetItemText(5,GetGadgetState(5),2), #PB_String_UpperCase  )
                  ;ButtonGadget(9, 500, 475, 100, 20, "Update Outline")
                
               
                 Case #PB_EventType_RightClick       :; Debug "Click with right mouse button"
                    ;DisplayPopupMenu(0, WindowID(0))
                 Case #PB_EventType_LeftDoubleClick  :; Debug "Double-click with left mouse button"
                                                      ;MessageRequester("Information", "Doubleclick: item"+Str(GetGadgetState(5))+", Text: "+GetGadgetText(5), 2)
                    
                 Case #PB_EventType_RightDoubleClick : ;Debug "Double-click with right mouse button"
               EndSelect

             Case 16 
               Select EventType()
                 Case #PB_EventType_LeftClick     
                   update$="UPDATE pinmap SET newpin='"+GetGadgetText(15)+"' WHERE component='"+GetGadgetText(14)+"' and pin='"+GetGadgetItemText(12,GetGadgetState(12),1)+"'"
                  
                   If DatabaseUpdate(2, update$)
                        AddGadgetItem(10, -1, "Update OK: "+update$)
                        SetGadgetItemText(12, Val(GetGadgetText(13)),GetGadgetText(15),2)
                        SetActiveGadget(12) 
                        SetGadgetState(12, Val(GetGadgetText(13))  )
                        SetGadgetItemState(12, Val(GetGadgetText(13)) ,#PB_ListIcon_Selected )
                   
                   Else
                        AddGadgetItem(10, -1, "Update failed: "+update$)
                   EndIf
                   
                   SetGadgetItemText(12, Val(GetGadgetText(13)),GetGadgetText(15),2)
                   SetActiveGadget(12) 
                   SetGadgetState(12, Val(GetGadgetText(13))  )
                   SetGadgetItemState(12, Val(GetGadgetText(13)) ,#PB_ListIcon_Selected )
       
                   
               EndSelect
              ;SetGadgetText(4, "ListIcon Gadget. Item selected: "+Str(GetGadgetState(5)))
          
          
          EndSelect

       
        Case #PB_Event_CloseWindow
             Quit = 1

    EndSelect

  Until Quit = 1
    If (IsDatabase(2)<>0) :  CloseDatabase(2) :   EndIf  
EndIf  ;for window open

; IDE Options = PureBasic 5.50 (Windows - x64)
; CursorPosition = 110
; FirstLine = 98
; Folding = --
; EnableXP
; Executable = DexVeeCadNetList.exe
; EnableUnicode